#include "LPC11XX.h"
#define KEIL_MCB1000_BOARD

/* Function declarations */
void LedOutputCfg(void);  // Set I/O pin connected to LED as output
void Timer0_Config(void); // Setup timer

/* Power down control bit definitions */
#define IRC_OUT_PD                      (0x1<<0)
#define IRC_PD                          (0x1<<1)
#define FLASH_PD                        (0x1<<2)
#define BOD_PD                          (0x1<<3)
#define ADC_PD                          (0x1<<4)
#define SYS_OSC_PD                      (0x1<<5)
#define WDT_OSC_PD                      (0x1<<6)
#define SYS_PLL_PD                      (0x1<<7)
#define USB_PLL_PD                      (0x1<<8)
#define MAIN_REGUL_PD                   (0x1<<9)
#define USB_PHY_PD                      (0x1<<10)
#define RESERVED1_PD                    (0x1<<11)
#define LP_REGUL_PD                     (0x1<<12)

int main(void)
{
  /* Use internal OSC for now */
  LPC_SYSCON->SYSAHBCLKDIV = 1;/* AHB clock same as system clock */

  /* Initialize hardware */
  LedOutputCfg();      /* Program LED I/O */
  Timer0_Config();     /* Program timer */

  /* Use port0_8 as wakeup source, i/o pin */
  LPC_IOCON->PIO0_8 = (2<<0); // Function set to MAT0
  /* Only edge trigger. Activation polarity on P0.8 is rising edge. */
  LPC_SYSCON->STARTAPRP0 = LPC_SYSCON->STARTAPRP0 | (1<<8);
  /* Clear all wakeup source */ 
  LPC_SYSCON->STARTRSRP0CLR = 0xFFFFFFFF;
  /* Enable Port 0.1 as wakeup source. */
  LPC_SYSCON->STARTERP0 = 1<<8;

  NVIC_ClearPendingIRQ(WAKEUP8_IRQn);
  NVIC_EnableIRQ(WAKEUP8_IRQn); // Enable wake up handler

  /* Turn off all other peripheral dividers */
  LPC_SYSCON->SSP0CLKDIV = 0;
  LPC_SYSCON->SSP1CLKDIV = 0;
  LPC_SYSCON->WDTCLKDIV = 0;
  LPC_SYSCON->SYSTICKCLKDIV = 0;

  /* Turn on the watchdog oscillator */
  LPC_SYSCON->PDRUNCFG   &= ~(1<<6);
  LPC_SYSCON->WDTOSCCTRL = (0x1<<5) | 0x1F; // Run watchdog at slow speed, 
                                            // with 1/64 prescale
  /* Switch MAINCLKSEL to Watchdog Oscillator */
  LPC_SYSCON->MAINCLKSEL = 2; // Set clock source to watchdog osc
  LPC_SYSCON->MAINCLKUEN = 0; // Enable update to watchdog oscillator
  LPC_SYSCON->MAINCLKUEN = 1;
  while (!(LPC_SYSCON->MAINCLKUEN & 0x01)); // wait to ensure update completed

  /* Enable flash and watchdog oscillator */
  LPC_SYSCON->PDRUNCFG = ~(WDT_OSC_PD | FLASH_PD | MAIN_REGUL_PD | LP_REGUL_PD);

  /* Copy current run mode power down configuration 
     to wake up configuration register so that
     current configuration is restored at wakeup */
  LPC_SYSCON->PDAWAKECFG = LPC_SYSCON->PDRUNCFG; 

  /* For deep sleep - retain power to flash, watchdog and reserved */
  LPC_SYSCON->PDSLEEPCFG = ~(FLASH_PD|WDT_OSC_PD|MAIN_REGUL_PD);

  LPC_TMR16B0->TCR = 1;      // Enable timer

  SCB->SCR = SCB->SCR | 0x4; // Turn on deep sleep feature
  while(1) {
    __WFI();    // Enter sleep mode
    };
} // end main

void Timer0_Config(void)
{ // Use 16-bit timer 0
  // Enable clock to 16-bit timer 0
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<7);

  LPC_TMR16B0->TCR = 2;        // Disable and reset timer
  LPC_TMR16B0->TCR = 0;        // Disable timer
  LPC_TMR16B0->PR  = 95;       // Prescaler set to 0 (TC increment every 96 cycle)
   // Watchdog oscillator will be configured as approx 9600 Hz.
   // By having prescale of 96, the timer increment every 10 ms
  LPC_TMR16B0->PC  = 0;        // Prescaler counter current value clear
  LPC_TMR16B0->TC  = 0;        // Timer counter current value clear
  LPC_TMR16B0->MR0 = 199;      // Match Register set to "200 - 1"
                               // because timer count at 100 Hz,
                               // match occur once every second
  LPC_TMR16B0->EMR = (0x2<<4); // Enable match output
  LPC_IOCON->PIO0_8= (2<<0);   // Set PIO0_8 to MAT0 output function
  LPC_TMR16B0->MCR = 2;        // When match MR0, reset counter
  return;
} // end Timer0_Config

void WAKEUP_IRQHandler(void)
{
  unsigned int regVal;
  int i,j;
  regVal = LPC_SYSCON->STARTSRP0;
  if ( regVal != 0 )
  {
	LPC_SYSCON->STARTRSRP0CLR = regVal;
  }
  /* Clear the timer match output to 0 */
  LPC_TMR16B0->EMR = LPC_TMR16B0->EMR & ~(1<<0);

#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  LPC_GPIO2->DIR = LPC_GPIO2->DIR | 0x1; // enable output
  for (i=0; i< 4; i++){
    LPC_GPIO2->MASKED_ACCESS[1] = ~LPC_GPIO2->MASKED_ACCESS[1]; // Toggle bit 0
    for (j=0; j<30;j++) { __ISB(); } // delay
    }
  LPC_GPIO2->DIR = LPC_GPIO2->DIR & ~(0x1); // turn off output to save power
#else
  // For LPCXpresso, use P0.7 for LED output
  LPC_GPIO0->DIR = LPC_GPIO0->DIR | (1<<7);  // enable output
  for (i=0; i< 4; i++){
    LPC_GPIO0->MASKED_ACCESS[1<<7] = ~LPC_GPIO0->MASKED_ACCESS[1<<7]; // Toggle bit 7 
    for (j=0; j<30;j++) { __ISB(); } // delay
    }
  LPC_GPIO0->DIR = LPC_GPIO0->DIR & ~(1<<7);  // turn off output to save power 
#endif  
  return;
}
void LedOutputCfg(void)
{
  // Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register)
  // and enable clock to GPIO (bit[6] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16) | (1<<6);
#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  // PIO2_0 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=DTR, 2=SSEL1)
  LPC_IOCON->PIO2_0 = (0<<5) + (0<<3) + (0x0);
  // Initial bit 0 output is 0
  LPC_GPIO2->MASKED_ACCESS[1] = 0;
  // Set pin 7 to 0 as output
  LPC_GPIO2->DIR = LPC_GPIO2->DIR | 0x1;
#else
  // For LPCXpresso, use P0.7 for LED output
  // PIO0_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=CTS)
  LPC_IOCON->PIO0_7 = (0x0) + (0<<3) + (0<<5);
  // Initial bit[7] output is 0
  LPC_GPIO0->MASKED_ACCESS[1<<7] = 0;
  // Set pin 7 as output
  LPC_GPIO0->DIR = LPC_GPIO0->DIR | (1<<7);
#endif
  return;
} // end LedOutputCfg

