#include "LPC11XX.h"
#define KEIL_MCB1000_BOARD

// Function declarations
void Set48MHzClock(void); // Program PLL to generate 48MHz clock
void LedOutputCfg(void);  // Set I/O pin connected to LED as output

int main(void)
{
  int i;
  // Switch Clock to 48MHz
  Set48MHzClock();
  // Initialize LED output
  LedOutputCfg();

  // Program SysTick timer at 1KHz.
  // At 48MHz, SysTick trigger every 48000 CPU cycles
  SysTick->LOAD = (48000-1); // Count from 47999 to 0
  SysTick->VAL  = 0;   // Clear SysTick value
  SysTick->CTRL = 0x5; // Enable, using core clock

  while(1){	 // Blink at 1Hz
    for (i=0;i<500;i++) { // Wait for 0.5 seconds
	  while ((SysTick->CTRL & 0x10000)==0); // Wait for counter underflow
	  }
#ifdef KEIL_MCB1000_BOARD
    // For Keil MCB1000, use P2.0 for LED output
    LPC_GPIO2->MASKED_ACCESS[1] = 0;
#else
    // For LPCXpresso, use P0.7 for LED output
    LPC_GPIO0->MASKED_ACCESS[1<<7] = (0<<7); // Clear bit 7
	// Alternatively, we can use "LPC_GPIO0->DATA = 0x00;"
	// as the other bits are not used    
#endif
    for (i=0;i<500;i++) { // Wait for 0.5 seconds
	  while ((SysTick->CTRL & 0x10000)==0); // Wait for counter underflow
	  }
#ifdef KEIL_MCB1000_BOARD
    // For Keil MCB1000, use P2.0 for LED output
    LPC_GPIO2->MASKED_ACCESS[1] = 1;
#else
    // For LPCXpresso, use P0.7 for LED output
    LPC_GPIO0->MASKED_ACCESS[1<<7] = (1<<7); // Set bit 7 
	// Alternatively, we can use "LPC_GPIO0->DATA = 0x80;"
	// as the other bits are not used
#endif
  } // end while
} // end main

// Switch LED signal to output port with no pull up or pulldown
void LedOutputCfg(void)
{
  // Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register)
  // and enable clock to GPIO (bit[6] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16) | (1<<6);
#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  // PIO2_0 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=DTR, 2=SSEL1)
  LPC_IOCON->PIO2_0 = (0<<5) + (0<<3) + (0x0);

  // Initial bit 0 output is 0
  LPC_GPIO2->MASKED_ACCESS[1] = 0;
  // Set pin 7 to 0 as output
  LPC_GPIO2->DIR = LPC_GPIO2->DIR | 0x1;
#else
  // For LPCXpresso, use P0.7 for LED output
  // PIO0_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=CTS)
  LPC_IOCON->PIO0_7 = (0x0) + (0<<3) + (0<<5);
  // Initial bit[7] output is 0
  LPC_GPIO0->MASKED_ACCESS[1<<7] = 0;
  // Set pin 7 as output
  LPC_GPIO0->DIR = LPC_GPIO0->DIR | (1<<7);
#endif
  return;
} // end LedOutputCfg

// Switch the CPU clock frequency to 48MHz
void Set48MHzClock(void)
{
  // Power up the PLL and System oscillator	
  // (clear the powerdown bits for PLL and System oscillator)
  LPC_SYSCON->PDRUNCFG = LPC_SYSCON->PDRUNCFG & 0xFFFFFF5F;

  // Select PLL source as crystal oscillator
  //  0 - IRC oscillator
  //  1 - System oscillator
  //  2 - WDT oscillator
  LPC_SYSCON->SYSPLLCLKSEL = 1;
  // Update SYSPLL setting (0->1 sequence)
  LPC_SYSCON->SYSPLLCLKUEN = 0;
  LPC_SYSCON->SYSPLLCLKUEN = 1;
  // Set PLL to 48MHz generate from 12MHz
  //  M = 48/12 = 4 (MSEL = 3)
  //  FCCO (must be between 156 to 320MHz, and is 2x, 4x, 8x or 16x of Clock) 
  //  Clock freq out selected as 192MHz
  //  P = 192MHz/48MHz/2 = 2 (PSEL = 1)
  //  bit[8]   - BYPASS
  //  bit[7]   - DIRECT
  //  bit[6:5] - PSEL  (1,2,4,8)
  //  bit[4:0] - MSEL  (1-32)
  LPC_SYSCON->SYSPLLCTRL = (3 + (1<<5)); // M = 4, P = 2
  // wait until PLL is locked
  while(LPC_SYSCON->SYSPLLSTAT == 0);
  // Switch main clock to PLL clock
  //   0 - IRC
  //   1 - Input clock to system PLL
  //   2 - WDT clock
  //   3 - System PLL output
  LPC_SYSCON->MAINCLKSEL = 3;
  // Update Main Clock Select setting (0->1 sequence)
  LPC_SYSCON->MAINCLKUEN = 0;
  LPC_SYSCON->MAINCLKUEN = 1;

  return;
} // end Set48MHzClock
