#include <RTL.h>
#include <LPC11XX.h>
#include <stdio.h>
#include "uart_io.h"
//#define KEIL_MCB1000_BOARD

OS_TID t_blinky; // Declare a task ID for LED blinky
OS_TID t_ctrl;   // Declare a task ID for main control

// Function declarations
void DialIOcfg(void);  // Configure Port 3 for dial interface
void LedOutputCfg(void);  // Set I/O pin connected to LED as output

// Global variable for communicating between main program and ISR
volatile int  DialValue;   // Dial value (0 to 0xFF)
short    int  last_state;  // Last state of I/O port signals

__task void blinky(void) {
  while(1) {   
#ifdef KEIL_MCB1000_BOARD
    // For Keil MCB1000, use P2.0 for LED output
    LPC_GPIO2->MASKED_ACCESS[1] = ~LPC_GPIO2->MASKED_ACCESS[1]; // Toggle bit 0
#else
    // For LPCXpresso, use P0.7 for LED output
    LPC_GPIO0->MASKED_ACCESS[1<<7] = ~LPC_GPIO0->MASKED_ACCESS[1<<7]; // Toggle bit 7 
#endif
    os_dly_wait (3 + (DialValue>>2));  // delay
    }
  }

__task void control(void) {
  DialValue=0;
  last_state=(LPC_GPIO3->DATA & 0xC)>>2; 
     // capture and save signal levels for next compare
  while(1) {   
    os_evt_wait_or(0x0001, 0xFFFF);
    printf("%d\n",DialValue);
    }
  }

__task void init (void) {
  t_blinky = os_tsk_create (blinky, 1); // Create a task "blinky" with priority 1
  t_ctrl   = os_tsk_create (control, 1); // Create a task "blinky" with priority 1
  os_tsk_delete_self ();
}  

int main(void)
{
  SystemInit();  // Switch Clock to 48MHz 
  UartConfig();  // Initialize UART
  DialIOcfg();   // IO port and interrupt setup
  LedOutputCfg();// Set I/O pin connected to LED
  printf ("\nDial test\n"); // Test message
  os_sys_init(init); // Initialize OS
} // end main

void DialIOcfg(void)
{ // The inputs are P3.2 and P3.3
  // Enable clock to GPIO block (bit[6] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<6);
  // PIO1_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=nDCD)
  LPC_IOCON->PIO3_2 = (0x0) + (0<<3) + (1<<5);
  // PIO1_6 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=RI)
  LPC_IOCON->PIO3_3 = (0x0) + (0<<3) + (1<<5);
  // Set direction of P3.2 and P3.3 as input
  LPC_GPIO3->DIR    = LPC_GPIO3->DIR & ~(0x0C); // Clear bit [3:2] to 0
  // Set interrupt of P3.2 and P3.3 as edge sensitive
  LPC_GPIO3->IS     = LPC_GPIO3->IS  & ~(0x0C); // Clear bit [3:2] to 0
  // Set interrupt of P3.2 and P3.3 for both rising and falling edge
  LPC_GPIO3->IBE    = LPC_GPIO3->IBE |  (0x0C); // Set bit [3:2] to 1
  // Set interrupt mask of P3.2 and P3.3 
  LPC_GPIO3->IE     = LPC_GPIO3->IE  |  (0x0C); // Set bit [3:2] to 1
  // Clear any previous interrupt of P3.2 and P3.3 
  LPC_GPIO3->IC     = LPC_GPIO3->IC  |  (0x0C); // write bit [3:2] to 1
  // Clear any previous occurred interrupt for port 3
  NVIC_ClearPendingIRQ(EINT3_IRQn);
  // Set priority of port 3 interrupt
  NVIC_SetPriority(EINT3_IRQn, 0);
  // Enable interrupt at NVIC
  NVIC_EnableIRQ(EINT3_IRQn);
  return;
}
// Switch LED signal (P0_7) to output port with no pull up or pulldown
void LedOutputCfg(void)
{
  // Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register)
  // and enable clock to GPIO (bit[6] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16) | (1<<6);
#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  // PIO2_0 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=DTR, 2=SSEL1)
  LPC_IOCON->PIO2_0 = (0<<5) + (0<<3) + (0x0);

  // Initial bit 0 output is 0
  LPC_GPIO2->MASKED_ACCESS[1] = 0;
  // Set pin 7 to 0 as output
  LPC_GPIO2->DIR = LPC_GPIO2->DIR | 0x1;
#else
  // For LPCXpresso, use P0.7 for LED output
  // PIO0_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=CTS)
  LPC_IOCON->PIO0_7 = (0x0) + (0<<3) + (0<<5);
  // Initial bit[7] output is 0
  LPC_GPIO0->MASKED_ACCESS[1<<7] = 0;
  // Set pin 7 as output
  LPC_GPIO0->DIR = LPC_GPIO0->DIR | (1<<7);
#endif
  return;
} // end LedOutputCfg

// Interrupt handler for port 3
void PIOINT3_IRQHandler(void)
{
  short int new_state;
  // Pattern for determine the direction of changes
  // Clock wise      pattern is 00 -> 01 -> 11 -> 10 -> 00 -> ...
  // Anti Clock wise pattern is 00 -> 10 -> 11 -> 01 -> 00 -> ...
  // After merging the new_state and last_state, clock wise can be
  // pattern b0100(4), b1101(13), b1011(11) and b0010(2)
  // anti-clockwise can be pattern b1000(8), b1110(14),b0111(7) and b0001(1)
  const signed char Pattern[] = { 0,-1,1,0,  1,0,0,-1,  -1,0,0,1,  0,1,-1,0}; 
  // Clear asserted interrupt P3.2 or P3.3 
  LPC_GPIO3->IC = LPC_GPIO3->MIS & (0x0C); // write bit [3:2] to 1
  // Extract bit 3 and 2 and combine with last state
  new_state = (LPC_GPIO3->DATA & 0xC) | last_state; 
  // Obtain increment/decrement info from new_state and calculate new DialValue
  DialValue = (DialValue + Pattern[new_state]) & 0xFF;
  // Save the current state for next time
  last_state  = (new_state & 0xC) >> 2;
  isr_evt_set(0x1, t_ctrl);// Send event
  return;
}
