#include "LPC11XX.h"
#include <stdio.h>

// Function declarations
void          DialIOcfg(void);  // Configure Port 3 for dial interface
void          UartConfig(void); // Uart configuration
unsigned char UartPutc(unsigned char my_ch);     // Uart character output

// Global variable for communicating between main program and ISR
volatile char DialUpdated; // Set to 1 if the dial value is updated
volatile int  DialValue;   // Dial value (0 to 0xFF)
short    int  last_state;  // Last state of I/O port signals

// Start of main program
int main(void)
{
  SystemInit();  // System Initialization

  DialUpdated=1; // Software variable initialization
  DialValue=0;
  last_state=(LPC_GPIO3->DATA & 0xC)>>2; 
     // capture and save signal levels for next compare

  UartConfig();  // Initialize UART
  DialIOcfg();   // IO port and interrupt setup
  printf ("\nDial test\n"); // Test message

  while(1){
    if (DialUpdated) {
	  DialUpdated = 0;
	  printf("%d\n",DialValue);
      } // end if
    } // end while
} // end main

void DialIOcfg(void)
{ // The inputs are P3.2 and P3.3
  // Enable clock to GPIO block (bit[6] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<6);
  // PIO1_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=nDCD)
  LPC_IOCON->PIO3_2 = (0x0) + (0<<3) + (1<<5);
  // PIO1_6 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=RI)
  LPC_IOCON->PIO3_3 = (0x0) + (0<<3) + (1<<5);
  // Set direction of P3.2 and P3.3 as input
  LPC_GPIO3->DIR    = LPC_GPIO3->DIR & ~(0x0C); // Clear bit [3:2] to 0
  // Set interrupt of P3.2 and P3.3 as edge sensitive
  LPC_GPIO3->IS     = LPC_GPIO3->IS  & ~(0x0C); // Clear bit [3:2] to 0
  // Set interrupt of P3.2 and P3.3 for both rising and falling edge
  LPC_GPIO3->IBE    = LPC_GPIO3->IBE |  (0x0C); // Set bit [3:2] to 1
  // Set interrupt mask of P3.2 and P3.3 
  LPC_GPIO3->IE     = LPC_GPIO3->IE  |  (0x0C); // Set bit [3:2] to 1
  // Clear any previous interrupt of P3.2 and P3.3 
  LPC_GPIO3->IC     = LPC_GPIO3->IC  |  (0x0C); // write bit [3:2] to 1
  // Clear any previous occurred interrupt for port 3
  NVIC_ClearPendingIRQ(EINT3_IRQn);
  // Set priority of port 3 interrupt
  NVIC_SetPriority(EINT3_IRQn, 0);
  // Enable interrupt at NVIC
  NVIC_EnableIRQ(EINT3_IRQn);
  return;
}
// Interrupt handler for port 3
void PIOINT3_IRQHandler(void)
{
  short int new_state;
  // Pattern for determine the direction of changes
  // Clock wise      pattern is 00 -> 01 -> 11 -> 10 -> 00 -> ...
  // Anti Clock wise pattern is 00 -> 10 -> 11 -> 01 -> 00 -> ...
  // After merging the new_state and last_state, clock wise can be
  // pattern b0100(4), b1101(13), b1011(11) and b0010(2)
  // anti-clockwise can be pattern b1000(8), b1110(14),b0111(7) and b0001(1)
  const signed char Pattern[] = { 0,-1,1,0,  1,0,0,-1,  -1,0,0,1,  0,1,-1,0}; 
  // Clear asserted interrupt P3.2 or P3.3 
  LPC_GPIO3->IC = LPC_GPIO3->MIS & (0x0C); // write bit [3:2] to 1
  // Extract bit 3 and 2 and combine with last state
  new_state = (LPC_GPIO3->DATA & 0xC) | last_state; 
  // Obtain increment/decrement info from new_state and calculate new DialValue
  DialValue = (DialValue + Pattern[new_state]) & 0xFF;
  // Save the current state for next time
  last_state  = (new_state & 0xC) >> 2;
  DialUpdated = 1;// Set software flag for display
  return;
}
void UartConfig(void)
{
  // UART interface are : PIO1_7 (TXD) and PIC1_6 (RXD)
  // Other UART signals (DTR, DSR, CTS, RTS, RI) are not used

  // Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16);

  // PIO1_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=TXD, 2=CT32B0_MAT1)
  LPC_IOCON->PIO1_7 = (0x1) + (0<<3) + (0<<5);
  // PIO1_6 IO input config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=RXD, 2=CT32B0_MAT0)
  LPC_IOCON->PIO1_6 = (0x1) + (2<<3) + (1<<5);

  // Enable clock to UART (bit[12] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<12);
  // UART_PCLK divide ratio = 1
  LPC_SYSCON->UARTCLKDIV = 1;

  // UART_PCLK = 48MHz, Baudrate = 38400, divide ratio = 1250
  // Line Control Register
  LPC_UART->LCR = (1<<7) |   // Enable access to Divisor Latches
    (0<<6) |   // Disable Break COntrol
    (0<<4) |   // Bit[5:4] parity select (odd, even, sticky-1, sticky-0)
    (0<<3) |   // parity disabled
    (0<<2) |   // 1 stop bit
    (3<<0);    // 8-bit data
  LPC_UART->DLL = 78;// Divisor Latch Least Significant Byte : 48MHz/38400/16=78.125
  LPC_UART->DLM = 0; // Divisor Latch Most Significant Byte  : 0
  LPC_UART->LCR = (0<<7) |   // Disable access to Divisor Latches
    (0<<6) |   // Disable Break COntrol
    (0<<4) |   // Bit[5:4] parity select (odd, even, sticky-1, sticky-0)
    (0<<3) |   // parity disabled
    (0<<2) |   // 1 stop bit
    (3<<0);    // 8-bit data

  LPC_UART->FCR = 1; // Enable FIFO

  return;
}
// Output a character
unsigned char UartPutc(unsigned char my_ch)
{
  if (my_ch == '\n') {
    while ((LPC_UART->LSR & (1<<5))==0); 
     // Wait if Transmit Holding register is not empty
    LPC_UART->THR = 13;  
     // Output carriage return (for Windows Hyperterminal)
    }
  while ((LPC_UART->LSR & (1<<5))==0); 
     // Wait if Transmit Holding register is not empty
  LPC_UART->THR = my_ch; 
     // write to transmit holding register

  if (my_ch == 13) {
    while ((LPC_UART->LSR & (1<<5))==0); 
     // Wait if Transmit Holding register is not empty
    LPC_UART->THR = 10;  
     // Output new line (for Windows Hyperterminal)
    }

  return (my_ch);
}
// Detect if new received data are available
// (only required if retarget.c is used because UartGetc is referenced in that file)
__inline int  UartGetRxDataAvail(void)
{
  return (LPC_UART->LSR & 0x1);
}
// Get a character from UART, if no data available then wait
// (only required if retarget.c is used because it is referenced in that file)
unsigned char UartGetc(void)
{
  while (UartGetRxDataAvail()==0); // wait if receive buffer empty
  return ((char)LPC_UART->RBR);
}
